import sys, os


setup_path = os.path.abspath(__file__)
bundle_path= os.path.dirname(setup_path)
bundles_path = os.path.dirname(bundle_path)
plugins_path = os.path.dirname(bundles_path)
trunk_path = os.path.dirname(plugins_path)

try:
    from elisa.core.utils.dist import elisa_setup
except ImportError:
    core_path = os.path.join(trunk_path, 'elisa-core')
    sys.path.insert(0, core_path)
    if 'elisa' in sys.modules:
        del sys.modules['elisa']
    from elisa.core.utils.dist import elisa_setup

from elisa.core import __version__

name = 'elisa-plugins-ugly'
description = "Elisa plugins bad bundle"
long_description = description
license = "GPL3"
version = __version__
author = "Elisa Team"
author_email = "contact@fluendo.com"

plugins_dir = os.path.join(os.getcwd(), 'elisa-plugins')
if not os.path.isdir(plugins_dir):
    plugins_dir = os.path.join(trunk_path, 'elisa-plugins')

elisa_setup(name, description, long_description, version, license,
            author, author_email, 'elisa', plugins_dir)
